using System.Diagnostics;
using System.Reflection;
using System.Runtime.InteropServices;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.GUI.MainScreen.UI;
using Luski.GUI.MainScreen.UI.PublicServers;
using Luski.GUI.StartPage.UI;
using Luski.net;
using Luski.net.Structures.Public;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.Desktop;
using Window = GraphicsManager.Window;

namespace Luski.GUI;

public class MainScreenWindow : Window
{
    private static readonly NativeWindowSettings Settings = new()
    {
        Title = "Luski Login",
        WindowBorder = WindowBorder.Fixed,
        APIVersion = new Version(3, 2),
        API = ContextAPI.OpenGL,
        StartFocused = true,
        Size = new Vector2i((int)(312 * Globals.Settings.Scale), (int)(545 * Globals.Settings.Scale)),
        Icon = Globals.Icon,
        SharedContext = null,
    };

    public TabControl? tc;
    private FlowLayout? channelpicker, friends, friend_request;
    private RoundedButton? FriendManagerBtn;
    Login login;
    CreateAccount ca;
    private static DebugProc DebugMessageDelegate = OnDebugMessage;

    private static void OnDebugMessage(
        DebugSource source, // Source of the debugging message.
        DebugType type, // Type of the debugging message.
        int id, // ID associated with the message.
        DebugSeverity severity, // Severity of the message.
        int length, // Length of the string in pMessage.
        IntPtr pMessage, // Pointer to message string.
        IntPtr pUserParam) // The pointer you gave to OpenGL, explained later.
    {
        // In order to access the string pointed to by pMessage, you can use Marshal
        // class to copy its contents to a C# string without unsafe code. You can
        // also use the new function Marshal.PtrToStringUTF8 since .NET Core 1.1.
        string message = Marshal.PtrToStringAnsi(pMessage, length);

        // The rest of the function is up to you to implement, however a debug output
        // is always useful.

        if (false)
        {
            switch (severity)
            {
                case DebugSeverity.DebugSeverityHigh:
                    Console.ForegroundColor = ConsoleColor.DarkRed;
                    Console.WriteLine("[{0} source={1} type={2} id={3}] {4}", severity, source, type, id, message);
                    Console.ResetColor();
                    break;
                case DebugSeverity.DebugSeverityMedium:
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine("[{0} source={1} type={2} id={3}] {4}", severity, source, type, id, message);
                    Console.ResetColor();
                    break;
                default:
                    Console.ForegroundColor = ConsoleColor.Green;
                    Console.WriteLine(message);
                    Console.ResetColor();
                    break;
            }
        }
    }

    public MainScreenWindow() : base(Settings)
    {
        VSync = VSyncMode.On;
        this.TryGetCurrentMonitorScale(out var h, out var v);
        GL.DebugMessageCallback(DebugMessageDelegate, IntPtr.Zero);
        GL.Enable(EnableCap.DebugOutput);
        Globals.DefaultFontFamly = FontFamily.LoadFontFamily("Noto Sans").Result;
        Globals.DefaultFont = FontInteraction.Load(Globals.DefaultFontFamly);
        Globals.DefaultFont.PixelHeight = (uint)(20 * Globals.Settings.Scale);
        Globals.DefaultFont.FontSize = FontSize.Regular;
        Globals.TopTimeFont = Globals.DefaultFont.Clone();
        Globals.TopTimeFont.PixelHeight = (uint)(12 * Globals.Settings.Scale);
        Globals.TopTimeFont.FontSize = FontSize.Regular;
        Globals.MessageFont = Globals.DefaultFont.Clone();
        Globals.MessageFont.PixelHeight = (uint)(17 * Globals.Settings.Scale);
        Globals.MessageFont.ExtraLinePixels = (uint)(5 * Globals.Settings.Scale);
        Globals.MessageFont.FontSize = FontSize.Regular;
        Globals.SmallTimeFont = Globals.DefaultFont.Clone();
        Globals.LuskiTexture = TextureManager.AddTexture(Tools.GetResourceStream(Assembly.GetExecutingAssembly(),
            "Luski.Resources.Textures.Luski.png"));
        CenterWindow(0);
        if ((Globals.Luski.MainServer is not null && !Globals.Luski.MainServer.IsLogedIn) ||
            !Globals.Luski.LoadedServers.Any(s => s.IsLogedIn))
        {
            Controls.Add(ca = new CreateAccount());
            ca.Visible = false;
            ca.ChangeToApp += LoginOnChangeToApp;
            Controls.Add(login = new Login());

            login.ChangeToApp += LoginOnChangeToApp;
            login.ChangeToCa += LoginOnChangeToCa;
            Thread t = new(_ =>
            {
                if (Globals.Luski.MainServer is not null) Globals.Luski.MainServer.EncryptionHandler.GenerateKeys();

            });
            t.Start();

        }

        WindowLoaded += OnWindowLoaded;
    }

    private Task OnWindowLoaded(Window arg)
    {
        if (Globals.UpdaterSettings.AutoUpdateCheck && new HttpClient()
                .GetAsync(
                    $"https://www.jacobtech.com/Updater/GetProgramVersion?directory=Luski&branch={Globals.UpdaterSettings.Branch.ToString()}&selfcontained={Globals.UpdaterSettings.SelfContained.ToString().ToLower()}&platform={Globals.UpdaterSettings.Platform}")
                .Result.Content.ReadAsStringAsync().Result !=
            FileVersionInfo.GetVersionInfo(Assembly.GetExecutingAssembly().Location).FileVersion)
        {
            var update = new UpdateWindow();
            var result = update.ShowDialogue(this);
            if (result == UpdateWindow.DialogueResult.Yes)
            {
                Globals.Download = true;
                Close();
            }
        }

        if ((Globals.Luski.MainServer is not null && Globals.Luski.MainServer.IsLogedIn) ||
            Globals.Luski.LoadedServers.Any(s => s.IsLogedIn)) Invoke(() => LoginOnChangeToApp());

        return Task.CompletedTask;
    }

    private Task LoginOnChangeToCa()
    {
        Title = "Create Account";
        ca.Visible = true;
        login.Visible = false;
        return Task.CompletedTask;
    }
    
    public FlowLayout ser;
    private UserControl? SerBox;
    public PublicChat pc;

    public async Task LoadPublicServer(PublicServer? Server)
    {
        GL.Enable(EnableCap.DepthTest);
        GL.DepthFunc(DepthFunction.Always);
        if (Server is null) return;
        BlockDraw = true;
        try
        {
            #region Box Init

            if (SerBox is null)
            {
                SerBox = new()
                {
                    Location = new(ser.Size.X, 0, 0),
                    Size = new(Size.X - ser.Size.X, Size.Y),
                    Anchor = ObjectAnchor.All,
                    BackgroundColor = new(20, 20, 20, 255)
                };
                Controls.Add(SerBox);
                SerBox.LoadToParent(this, this);
            }

            SerBox.Controls.Clear();

            #endregion
            
            #region Channel Selector Init

            SocketChannel current_channel = await Server.User.GetSelectedChannel(CancellationToken.None);
            Channel.seltec = Globals.ms.TextureManager.AddTexture(
                Tools.GetResourceStream(Assembly.GetExecutingAssembly(),
                    "Luski.Resources.Textures.RoundedRectangle.png"));
            List<SocketCategory> parents = new();
            SocketCategory? cur = await current_channel.GetParent();
            while (cur is not null)
            {
                parents.Add(cur);
                cur = await cur.GetParent();
            }

            parents.Reverse();
            ChannelSelector cs = new(parents[0])
            {
                BackgroundColor = new(34, 34, 34, 255),
                Size = new((int)(307 * Globals.Settings.Scale), SerBox.Size.Y - 106),
                Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom
            };
            parents.RemoveAt(0);
            SerBox.Controls.Add(cs);
            cs.ForceDistanceUpdate(SerBox);

            #endregion

            #region Chat Init

            pc = new()
            {
                Anchor = ObjectAnchor.All,
                Location = new(cs.Size.X, 0, 0),
                Size = new(SerBox.Size.X - cs.Size.X, SerBox.Size.Y),
            };
            SerBox.Controls.Add(pc);
            pc.LoadToParent(SerBox, this);
            pc.ForceDistanceUpdate();
            pc.MessageFlow.ForceDistanceUpdate(pc);

            #endregion

            #region Channel Selector Loader

            _ = cs.Load(current_channel, parents);

            #endregion

            #region User Icon

            Role[] ra = await Server.User.GetRoles();
            Color c = ra[0].Color;
            Color4 c4 = new(c.R, c.G, c.B, c.A);
            Rectangle u = new Rectangle(TextureManager.AddTexture(Tools.GetResourceStream(
                Assembly.GetExecutingAssembly(),
                "Luski.Resources.Textures.Status.png")))
            {
                Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left, Size = new((int)(46 * Globals.Settings.Scale)),
                Location = new((int)(4 * Globals.Settings.Scale), (int)(812 * Globals.Settings.Scale), 0)
            };
            u.Shader = Rectangle.DefaultAlphaTextureShader[Context];
            u.Textures.Add(await Server.User.GetIcon());
            SerBox.Controls.Add(u);
            u.LoadToParent(SerBox, this);
            u.ForceDistanceUpdate();
            Label ul = new Label(Globals.DefaultFont)
            {
                Anchor = u.Anchor,
                Text = Server.User.DisplayName,
                Color = c4
            };

            ul.Location = new(u.Location.X + u.Size.X + (int)(5 * Globals.Settings.Scale),
                (u.Location.Y + (u.Size.Y / 2) - (ul.PostiveTrueHeight / 2) - ul.Size.Y + ul.TrueHeight), 0);
            SerBox.Controls.Add(ul);

            #endregion
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }

        BlockDraw = false;
    }

    public async Task LoadMainServer(MainServer Server)
    {
        if (SerBox is null)
            SerBox = new()
            {
                Location = new(ser.Size.X, 0, 0),
                Size = new(Size.X - ser.Size.X, Size.Y),
                Anchor = ObjectAnchor.All
            };
        Controls.Add(SerBox);
        SerBox.ForceDistanceUpdate();
        SerBox.Controls.Clear();
    }

    private async Task<Task> LoginOnChangeToApp()
    {
        Controls.Clear();
        BlockDraw = true;
        Title = "Luski";
        Size = new((int)(1332 * Globals.Settings.Scale), (int)(866 * Globals.Settings.Scale));
        DateTime start = DateTime.Now;
        CenterWindow(0);
        DateTime start1 = DateTime.Now;
        WindowBorder = WindowBorder.Resizable;
        BackgroundColor = new Color4(34, 34, 34, 255);

        Controls.Add(ser = new FlowLayout()
        {
            BackgroundColor = new(26, 26, 26, 255),
            Size = new((int)(68 * Globals.Settings.Scale), (int)(868 * Globals.Settings.Scale)),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom,
            Location = new(0,0,0)
        });
        DrawFrame();

        foreach (PublicServer pser in Globals.Luski.LoadedServers)
        {
            ser.Controls.Add(new ServerIcon<PublicServer>(pser));
        }

        AddServerIcon asi = new();
        asi.Button.Clicked += AddButtonClicked;
        ser.Controls.Add(asi);

        await (ser.Controls[0] as ServerIcon<PublicServer>)!.LoadServer();

        MainShow += OnMainShow;
        return Task.CompletedTask;
    }
    
    private Task AddButtonClicked(IRenderObject arg)
    {
        AddServerOverlay aso = new();
        aso.Clicked += AsoOnClicked;
        Controls.Add(aso);
        TryDraw();
        return Task.CompletedTask;
    }

    private Task AsoOnClicked(IRenderObject arg)
    {
        Globals.ms.Controls.Remove(arg);
        Globals.ms.DrawFrame();
        AddServerOverlay aso = (arg as AddServerOverlay)!;
        aso.Clicked -= AsoOnClicked;
        aso.Clean();
        aso = null!;
        return Task.CompletedTask;
    }

    private Task OnMainShow()
    {
        if (Globals.Luski.MainServer is not null && Globals.Luski.MainServer.IsLogedIn)
        {
            Globals.Luski.MainServer.OnError += LuskiOnOnError;
        }

        return Task.CompletedTask;
    }

    public event Func<Task>? MainShow;

    private Task LuskiOnOnError(Exception arg)
    {
        Console.WriteLine(arg);
        return Task.CompletedTask;
    }

}